package com.hellokoding.jpa;

import com.hellokoding.jpa.model.Book;
import com.hellokoding.jpa.model.BookPublisher;
import com.hellokoding.jpa.model.Publisher;
import com.hellokoding.jpa.repository.BookPublisherRepository;
import com.hellokoding.jpa.repository.BookRepository;
import com.hellokoding.jpa.repository.PublisherRepository;
import com.hellokoding.jpa.service.BookService;
import com.vaadin.annotations.Theme;
import com.vaadin.server.VaadinRequest;
import com.vaadin.spring.annotation.SpringUI;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

@Theme(ValoTheme.THEME_NAME)
@SpringUI
public class HelloJpaApplicationUI extends UI {


    BookRepository bookRepository;


    PublisherRepository publisherRepository;

    BookPublisherRepository bookPublisherRepository;

    @Autowired
    BookService bookService;

    ComboBox bookComboBox = new ComboBox();
    ComboBox newPublisherComboBox = new ComboBox();
    ComboBox oldPublisherComboBox = new ComboBox();

    public HelloJpaApplicationUI(BookRepository bookRepository, PublisherRepository publisherRepository, BookPublisherRepository bookPublisherRepository) {
        this.bookRepository = bookRepository;
        this.publisherRepository = publisherRepository;
        this.bookPublisherRepository = bookPublisherRepository;

    }

    @Override
    protected void init(VaadinRequest vaadinRequest) {

        HorizontalLayout layout = new HorizontalLayout();
        layout.setSpacing(true);
        layout.setMargin(true);


        bookComboBox.addItems(bookRepository.findAll());

        bookComboBox.addValueChangeListener(valueChangeEvent -> {
            updateComboxes();

        });

        bookComboBox.setNewItemsAllowed(false);
        bookComboBox.setTextInputAllowed(false);

        newPublisherComboBox.setNewItemsAllowed(false);
        newPublisherComboBox.setTextInputAllowed(false);

        oldPublisherComboBox.setNewItemsAllowed(false);
        oldPublisherComboBox.setTextInputAllowed(false);

        Button addButton = new Button("Add");
        addButton.setStyleName("primary");
        addButton.addClickListener(clickEvent -> {
            Book book = (Book) bookComboBox.getValue();
            if(book != null){
                Publisher publisher = (Publisher) newPublisherComboBox.getValue();
                if(publisher != null){
                    BookPublisher bookPublisher = bookPublisherRepository.findByBookAndPublisher(book, publisher);
                    if(bookPublisher == null){
                        bookPublisher = new BookPublisher(book,publisher);
                        bookPublisherRepository.save(bookPublisher);
                    }
                    updateComboxes();
                }
            }
        });

        Button removeButton = new Button("Remove");
        removeButton.setStyleName("danger");
        removeButton.addClickListener(clickEvent -> {
            Book book = (Book) bookComboBox.getValue();
            if(book != null){
                Publisher publisher = (Publisher) oldPublisherComboBox.getValue();
                if(publisher != null){
                    BookPublisher bookPublisher = bookPublisherRepository.findByBookAndPublisher(book, publisher);
                    if(bookPublisher != null){
                        bookPublisherRepository.delete(bookPublisher);
                    }
                    updateComboxes();
                }
            }
        });


        layout.addComponents(bookComboBox, newPublisherComboBox, addButton, oldPublisherComboBox, removeButton);


        setContent(layout);
    }

    private void updateComboxes() {
        oldPublisherComboBox.removeAllItems();
        newPublisherComboBox.removeAllItems();
        Book book = (Book) bookComboBox.getValue();
        if(book != null) {
            book = bookRepository.findByName(book.getName());
            List<Publisher> publisherList = publisherRepository.findAll();
            this.bookService.loadFully(book);
            List<Publisher> oldPublisherList = book.getPublishers();
            oldPublisherComboBox.addItems(oldPublisherList);
            List<?> newPubliserList = (List) CollectionUtils.disjunction(publisherList, oldPublisherList); //List<?> to make it type safe
            newPublisherComboBox.addItems(newPubliserList);
        }
    }
}
